
# q-expansion of newform 338.2.a.e, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 338
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [-1], [4], [4], [0], [3], [0], [-4], [-1], [4], [3], [-9], [-8], [-8], [-9], [-4], [7], [4], [-8], [11], [-4], [0], [-6], [2], [7], [-8], [-4], [-2], [-1], [-8], [20], [-9], [16], [15], [12], [11], [8], [12], [14], [-24], [-21], [-20], [11], [6], [-4], [20], [12], [24], [6], [10], [-12], [3], [-12], [15], [-24], [-18], [-20], [-9], [-5], [28], [-5], [-28], [-24], [-22], [3], [8], [23], [12], [-2], [7], [24], [-28], [-13], [-8], [0], [-9], [14], [3], [31], [12], [-5], [-24], [-5], [8], [24], [34], [31], [-33], [16], [20], [0], [-16], [-8], [-32], [16], [43], [39], [36], [-25], [16], [39], [-4], [10], [12], [39], [-16], [-1], [-44], [19], [0], [11], [-29], [4], [-8], [19], [44], [28], [-18], [48], [-49], [-29], [6], [-44], [-20], [30], [-13], [-28], [28], [-1], [-24], [-8], [24], [6], [42], [-30], [38], [-4], [30], [39], [-28], [38], [12], [-12], [7], [-40], [7], [-17], [20], [52], [15], [-9], [16], [-20], [-24], [-12], [-48], [-21], [-21], [46], [24], [-22], [-40], [0], [-45], [-52], [-24], [-25]]
